/*
 * main.h
 *
 * PWLib application header file for Dump PDU
 *
 * Copyright 1999 Equivalence
 *
 * $Log: main.h,v $
 * Revision 1.2  1999/08/31 13:30:20  robertj
 * Added gatekeeper support.
 *
 * Revision 1.1  1999/08/08 10:05:40  robertj
 * Added PDU dump program.
 *
 * Revision 1.3  1999/05/24 03:03:08  robertj
 * Fixed include paths after restructure (much belated).
 *
 * Revision 1.2  1999/01/26 06:35:08  robertj
 * Fixed $LOG$ variable in template files
 *
 */

#ifndef _DumpPDU_MAIN_H
#define _DumpPDU_MAIN_H


#include "q931.h"
#include "h225.h"
#include "h245.h"
#include "x224.h"
#include "mcspdu.h"


class TcpStreamKey : public PObject
{
    PCLASSINFO(TcpStreamKey, PObject);
  public:
    TcpStreamKey();
    TcpStreamKey(PIPSocket::Address tx_ip,
                 unsigned           tx_port,
                 PIPSocket::Address rx_ip,
                 unsigned           rx_port);

    virtual Comparison Compare(const PObject & obj) const;
    virtual PINDEX HashFunction() const;
    virtual PObject * Clone() const;
    virtual void ReadFrom(istream & strm);

    void ReverseCopy(const TcpStreamKey & key);

    PIPSocket::Address tx_ip;
    unsigned           tx_port;
    PIPSocket::Address rx_ip;
    unsigned           rx_port;
    unsigned           pdu_count;
};


PLIST(TcpStreamList, TcpStreamKey);


class TcpStreamData : public PBYTEArray
{
    PCLASSINFO(TcpStreamData, PBYTEArray);
  public:
    TcpStreamData();

    virtual void ReadFrom(istream & strm);

    BOOL HasTPKT();
    PBYTEArray GetTPKT();

  private:
    PINDEX position;
};


PDICTIONARY(TcpStreamDict, TcpStreamKey, TcpStreamData);


class H225withH245 : public H225_H323_UserInformation
{
    PCLASSINFO(H225withH245, H225_H323_UserInformation);
  public:
    void PrintOn(ostream & output) const;
};


class Q931withH225 : public Q931
{
    PCLASSINFO(Q931withH225, Q931);
  public:
    BOOL Decode(const PBYTEArray & rawData);
    void PrintOn(ostream & output) const;

    BOOL hasH225;
    H225withH245 h225;
};



class DumpPDU : public PProcess
{
  PCLASSINFO(DumpPDU, PProcess)

  public:
    DumpPDU();
    void Main();
};


#endif  // _DumpPDU_MAIN_H


// End of File ///////////////////////////////////////////////////////////////
