/*
 * main.h
 *
 * PWLib application header file for OpenPhone
 *
 * Copyright 1999 Equivalence
 *
 * $Log: main.h,v $
 * Revision 1.1  1999/08/25 05:16:11  robertj
 * GUI test application.
 *
 * Revision 1.3  1999/05/24 03:03:08  robertj
 * Fixed include paths after restructure (much belated).
 *
 * Revision 1.2  1999/01/26 06:35:08  robertj
 * Fixed $LOG$ variable in template files
 *
 */

#ifndef _OpenPhone_MAIN_H
#define _OpenPhone_MAIN_H

#include "h323.h"

class MainWindow;


///////////////////////////////////////////////////////

class OpenPhoneEndPoint : public H323EndPoint
{
  PCLASSINFO(OpenPhoneEndPoint, H323EndPoint);

  public:
    OpenPhoneEndPoint(MainWindow * mainWindow);

    // overrides from H323EndPoint
    void OnConnectionEstablished(H323Connection &, const PString & token);
    void OnConnectionCleared    (H323Connection &, const PString &);
    H323Connection * CreateConnection(unsigned callReference);

    // new functions
    virtual void Initialise(PConfig & config);

  protected:
    MainWindow * mainWindow;
};


///////////////////////////////////////////////////////

class OpenPhoneConnection : public H323Connection
{
  PCLASSINFO(OpenPhoneConnection, H323Connection);

  public:
    OpenPhoneConnection(
      OpenPhoneEndPoint & mgr,
      unsigned callReference,
      MainWindow * mainWindow
    );

    // overrides from H323Connection
    BOOL OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
    BOOL OnAlerting(const H323SignalPDU &, const PString &);

  protected:
    MainWindow * mainWindow;
};

///////////////////////////////////////////////////////

class MainWindow : public PTopLevelWindow
{
  PCLASSINFO(MainWindow, PTopLevelWindow)
  
  public:
    enum {
      Listening,
      Calling,
      InCall,
      Answering,
      HangingUp,
      Bogus
    };

    MainWindow(PArgList & args);
    ~MainWindow();

    PDECLARE_COMMAND("MakeCall",     MainWindow, CallCmd);
    PDECLARE_COMMAND("HangupCall",   MainWindow, HangupCmd);

    PDECLARE_COMMAND("AcceptCall",   MainWindow, AcceptCallCmd);
    PDECLARE_COMMAND("RefuseCall",   MainWindow, RefuseCallCmd);
    PDECLARE_COMMAND("ExitCmd",      MainWindow, ExitCmd);
//    PDECLARE_COMMAND("AutoAnswer",   MainWindow, AutoAnswerCmd);
//    PDECLARE_COMMAND("UseGateway",   MainWindow, UseGatewayCmd);
//    PDECLARE_COMMAND("ConfigCodecs", MainWindow, ConfigCodecsCmd);

    PDECLARE_NOTIFIER(PTextButton, MainWindow, OnCallButton);

    void SetStatus(const PString & str) { statusWindow->SetName(str); }

    void OnMenuStartSelect();

    void OnConnectionEstablished(const PString & token);
    void OnConnectionCleared();
    BOOL OnAnswerCall(const PString & caller);
    BOOL OnAlerting(const PString &);

  protected:
    // user interface vars
    PStaticText * statusWindow;
    PTextButton * callButton;

    OpenPhoneEndPoint * endPoint;

    int     callState;
    PString currentCallToken;
    PString remoteParty;
};


class OpenPhone : public PApplication
{
  PCLASSINFO(OpenPhone, PApplication)

  public:
    OpenPhone();
    void Main();
};


#endif  // _OpenPhone_MAIN_H


// End of File ///////////////////////////////////////////////////////////////
