/*
 * main.h
 *
 * PWLib application header file for Voxilla
 *
 * A H.323 "net telephone" application.
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions of this code were written with the assisance of funding from
 * Vovida Networks, Inc. http://www.vovida.com.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: main.h,v $
 * Revision 1.19  1999/08/31 12:34:19  robertj
 * Added gatekeeper support.
 *
 * Revision 1.18  1999/08/25 05:17:44  robertj
 * Simplified application structure (better for tutorial purposes)
 *
 * Revision 1.17  1999/07/26 07:06:12  craigs
 * Added support for QuickNet Cards
 *
 * Revision 1.16  1999/07/16 16:05:14  robertj
 * Added more run time options.
 *
 * Revision 1.15  1999/07/13 09:53:24  robertj
 * Fixed some problems with jitter buffer and added more debugging.
 *
 * Revision 1.14  1999/07/13 02:50:58  craigs
 * Changed semantics of SetPlayDevice/SetRecordDevice, only descendent
 *    endpoint assumes PSoundChannel devices for audio codec.
 *
 * Revision 1.13  1999/07/09 06:09:52  robertj
 * Major implementation. An ENORMOUS amount of stuff added everywhere.
 *
 * Revision 1.12  1999/06/25 16:21:02  robertj
 * Restructure of directories
 *
 * Revision 1.11  1999/06/22 13:42:31  robertj
 * Added user question on listener version to accept incoming calls.
 *
 * Revision 1.10  1999/06/09 05:26:19  robertj
 * Major restructuring of classes.
 *
 * Revision 1.9  1999/06/06 06:06:35  robertj
 * Changes for new ASN compiler and v2 protocol ASN files.
 *
 * Revision 1.8  1999/05/28 12:39:58  craigs
 * Added ability to select sound device at run time
 *
 * Revision 1.7  1999/05/22 12:51:58  craigs
 * Fixed to work with new Linux implementation
 *
 * Revision 1.6  1999/04/26 06:14:46  craigs
 * Initial implementation for RTP decoding and lots of stuff
 * As a whole, these changes are called "First Noise"
 *
 * Revision 1.5  1999/02/23 11:04:28  robertj
 * Added capability to make outgoing call.
 *
 * Revision 1.4  1999/01/16 08:05:00  robertj
 * Moved h323 code to a library
 *
 * Revision 1.3  1999/01/16 01:31:36  robertj
 * Major implementation.
 *
 * Revision 1.2  1998/12/14 10:15:30  robertj
 * Fixed leak of sockets
 *
 * Revision 1.1  1998/12/14 09:13:23  robertj
 * Initial revision
 *
 */

#ifndef _Voxilla_MAIN_H
#define _Voxilla_MAIN_H

#include "h323.h"

class Voxilla : public PProcess
{
    PCLASSINFO(Voxilla, PProcess)
  public:
    Voxilla();
    ~Voxilla();

    void Main();
};


class MyH323EndPoint : public H323EndPoint
{
    PCLASSINFO(MyH323EndPoint, H323EndPoint);
  public:
    // overrides from H323EndPoint
    virtual H323Connection * CreateConnection(unsigned callReference);
    virtual void OnConnectionEstablished(H323Connection &, const PString &);
    virtual void OnConnectionCleared(H323Connection &, const PString &);
    virtual PChannel * OpenAudioChannel(BOOL, unsigned);

    // new functions
    BOOL Initialise(PArgList &);
    virtual BOOL InitialiseCodecs(PArgList &);
    virtual void ListenForIncomingCalls();
    virtual void MakeOutgoingCall(const PString &, const PString &);
    virtual void AwaitTermination();

    void HandleUserInterface();

  protected:
    // only used internally
    BOOL SetSoundDevice(PArgList &, const char *, PSoundChannel::Directions);


    BOOL noFastStart;

    PString currentCallToken;

    enum {
      uiWaitingForCall,
      uiAnsweringCall,
      uiAnsweredCall,
      uiMakingCall
    } uiState;

    PSyncPoint exitFlag;
    PSyncPoint answerFlag;

  friend class MyH323Connection;
};


class MyH323Connection : public H323Connection
{
    PCLASSINFO(MyH323Connection, H323Connection);

  public:
    MyH323Connection(MyH323EndPoint &, unsigned, BOOL);

    // overrides from H323Connection
    BOOL OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);
    void OnSelectLogicalChannels();
    BOOL OnOpenLogicalChannel(const H245_OpenLogicalChannel &, H245_OpenLogicalChannelAck &, unsigned &);
    BOOL OnStartLogicalChannel(H323Channel &);
    BOOL OnAlerting(const H323SignalPDU &, const PString &);
    void OnUserInputString(const PString &);

  protected:
    MyH323EndPoint & myEndpoint;
    BOOL             noFastStart;
};


class UserInterfaceThread : public PThread
{
    PCLASSINFO(UserInterfaceThread, PThread);
  public:
    UserInterfaceThread(MyH323EndPoint & end)
      : PThread(1000, AutoDeleteThread), endpoint(end) { Resume(); }
    void Main()
      { endpoint.HandleUserInterface(); }
  protected:
    MyH323EndPoint & endpoint;
};


#endif  // _Voxilla_MAIN_H


// End of File ///////////////////////////////////////////////////////////////
