/*
 * xjack.h
 *
 * PWLib application header file for QuickPhone
 *
 * A H.323 "net telephone" application.
 *
 * Copyright (c) 1993-1998 Equivalence Pty. Ltd.
 *
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.0 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Portable Windows Library.
 *
 * The Initial Developer of the Original Code is Equivalence Pty. Ltd.
 *
 * Portions of this code were written with the assisance of funding from
 * Vovida Networks, Inc. http://www.vovida.com.
 *
 * Portions are Copyright (C) 1993 Free Software Foundation, Inc.
 * All Rights Reserved.
 *
 * Contributor(s): ______________________________________.
 *
 * $Log: xjack.h,v $
 * Revision 1.3  1999/08/27 02:12:25  robertj
 * Added missing AwaitingTermiantion override.
 *
 * Revision 1.2  1999/08/25 05:17:44  robertj
 * Simplified application structure (better for tutorial purposes)
 *
 * Revision 1.1  1999/07/26 07:06:53  craigs
 * Initial version with support for Quicknet cards
 *
 */

#ifndef _Voxilla_XJACK_H
#define _Voxilla_XJACK_H

#include "h323.h"
#include "main.h"


class XJackH323EndPoint : public MyH323EndPoint
{
  PCLASSINFO(XJackH323EndPoint, MyH323EndPoint);

  public:
    // Override from H323EndPoint
    H323Connection * CreateConnection(unsigned callReference);

    // Override from MyH323EndPoint
    virtual BOOL InitialiseCodecs(PArgList &);
    virtual void ListenForIncomingCalls();
    virtual void MakeOutgoingCall(const PString &, const PString &);
    virtual void AwaitTermination();

    // New functions
    BOOL RingHandset(int num);
    BOOL IsOnHook();

    int GetHandle()           { return ixj_handle; }
    int GetOpenCount(int adj) { return openCount += adj; }
    PString GetDeviceName()   { return deviceName; }

    int Read(BYTE * buffer, unsigned len);
    int Write(const BYTE * buffer, unsigned len);
    void ReadWrite();

  protected:
    int ixj_handle;
    int openCount;
    PString deviceName;

    PMutex mutex;
    BYTE * readBuffer;
    const BYTE * writeBuffer;
    int readLen, writeLen;
    int readStat, writeStat;

  friend class XJackH323Connection;
};


class XJackH323Connection : public MyH323Connection
{
  PCLASSINFO(XJackH323Connection, MyH323Connection);

  public:
    XJackH323Connection(XJackH323EndPoint & mgr, unsigned callReference, BOOL _noFastStart);

    BOOL OnAnswerCall(const PString &, const H323SignalPDU &, H323SignalPDU &);

  protected:
    XJackH323EndPoint & xjEndpoint;
};


class XJackCodec : public H323Codec
{
  PCLASSINFO(XJackCodec, H323Codec);

  enum { STARTUPCOUNT = 10 };

  enum {
    Codec_uLaw64k,
    Codec_G7231,
    Codec_Num
  };

  public:
    XJackCodec(Direction dir, PINDEX codecIndex);
    ~XJackCodec();

    unsigned GetMaxFrameSize() const { return 12*sampleBlockSize; }
    unsigned GetFrameRate() const { return sampleCount; }
    unsigned GetTimeUnits() const { return 8; }
    BOOL Read(BYTE * buffer, unsigned & length);
    BOOL Write(const BYTE * buffer, unsigned length, unsigned & frames);
    BOOL Open(H323Connection & connection);
    void Close();

  protected:
    int handle;
    unsigned codecIndex;
    unsigned sampleBlockSize;
    XJackH323EndPoint * xjackEndp;
    int sampleCount;
    PFile rtpFile;
};


class XJackCapability : public H323AudioCapability
{
  PCLASSINFO(XJackCapability, H323AudioCapability)

  public:
    XJackCapability(PINDEX codecIndex);
    unsigned GetSubType() const   { return subType; }
    BOOL OnSendingPDU(H245_AudioCapability & cap) const;
    virtual BOOL EncodeSendingPDU(H245_AudioCapability & cap) const = 0;

    H323Codec * CreateCodec(H323Codec::Direction direction) const;

    void PrintOn(ostream & strm) const;

  protected:
    PINDEX codecIndex;
    unsigned subType;
};


class XJackG7231Capability : public XJackCapability
{
  PCLASSINFO(XJackG7231Capability, XJackCapability)
  public:
    XJackG7231Capability()
      : XJackCapability(XJackCodec::Codec_G7231) {}
    BOOL EncodeSendingPDU(H245_AudioCapability & cap) const;
    BOOL OnReceivedPDU(const H245_AudioCapability & pdu);

};


class XJackG711uLaw64kCapability : public XJackCapability
{
  PCLASSINFO(XJackG711uLaw64kCapability, XJackCapability)
  public:
    XJackG711uLaw64kCapability()
      : XJackCapability(XJackCodec::Codec_uLaw64k) {}
    BOOL EncodeSendingPDU(H245_AudioCapability & cap) const;
    BOOL OnReceivedPDU(const H245_AudioCapability & pdu);
};



#endif  // _Voxilla_XJACK_H


// End of File ///////////////////////////////////////////////////////////////
